"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SmartInputFetcher = void 0;
var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));
var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));
var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));
var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _lodash = _interopRequireDefault(require("lodash.debounce"));
var _lodash2 = _interopRequireDefault(require("lodash.throttle"));
var _ulid = require("ulid");
/**
 * Responsible for fetching completions from a model and adding them to the input.
 */
var SmartInputFetcher = /*#__PURE__*/function () {
  /**
   * sequenceId is used to ensure that we don't add completions from a previous fetch
   */

  /**
   * currentCompletion is used to track the latest text that was sent to the model for fetching completions, and the
   * completion that was returned (if any ~ we may be waiting for it).
   */

  function SmartInputFetcher(addCompletion, modelId, mockCompletion) {
    var _this = this;
    (0, _classCallCheck2["default"])(this, SmartInputFetcher);
    (0, _defineProperty2["default"])(this, "sequenceId", 0);
    (0, _defineProperty2["default"])(this, "sessionId", (0, _ulid.ulid)());
    (0, _defineProperty2["default"])(this, "fetchCache", {});
    (0, _defineProperty2["default"])(this, "alreadySeen", new Set());
    (0, _defineProperty2["default"])(this, "alreadyAccepted", new Set());
    (0, _defineProperty2["default"])(this, "currentCompletion", null);
    (0, _defineProperty2["default"])(this, "fetchCompletion", (0, _lodash["default"])( /*#__PURE__*/function () {
      var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(text, context) {
        var _this$currentCompleti, _this$currentCompleti3;
        var seqId, currExpectedText, cachedVal, _completion, res, data;
        return _regenerator["default"].wrap(function _callee$(_context) {
          while (1) switch (_context.prev = _context.next) {
            case 0:
              if (!(text.length === 0)) {
                _context.next = 2;
                break;
              }
              return _context.abrupt("return", null);
            case 2:
              seqId = ++_this.sequenceId; // if a completion was already added, and the user is typing the same characters as the current completion, don't fetch
              if (!((_this$currentCompleti = _this.currentCompletion) !== null && _this$currentCompleti !== void 0 && _this$currentCompleti.completion)) {
                _context.next = 7;
                break;
              }
              currExpectedText = _this.currentCompletion.text + _this.currentCompletion.completion;
              if (!currExpectedText.startsWith(text)) {
                _context.next = 7;
                break;
              }
              return _context.abrupt("return", null);
            case 7:
              // update the current completion
              _this.currentCompletion = {
                text: text,
                sequenceId: seqId
              };

              // if we already have a completion for this text, don't fetch, but do return it
              cachedVal = _this.fetchCache[text];
              if (!cachedVal) {
                _context.next = 13;
                break;
              }
              // don't fetch the same text twice
              _this.addCompletion(text, cachedVal);
              _this.currentCompletion.completion = cachedVal.completion;
              return _context.abrupt("return", cachedVal);
            case 13:
              if (!_this.mockCompletion) {
                _context.next = 20;
                break;
              }
              _completion = _this.mockCompletion(text);
              if (!_completion) {
                _context.next = 19;
                break;
              }
              _context.next = 18;
              return new Promise(function (resolve) {
                setTimeout(function () {
                  var _this$currentCompleti2;
                  var c = {
                    input: text,
                    sid: (0, _ulid.ulid)(),
                    completion: _completion
                  };
                  _this.fetchCache[text] = c;
                  if (((_this$currentCompleti2 = _this.currentCompletion) === null || _this$currentCompleti2 === void 0 ? void 0 : _this$currentCompleti2.sequenceId) === seqId) {
                    _this.addCompletion(text, c);
                    _this.currentCompletion.completion = _completion;
                  }
                  resolve(c);
                }, getRandomInt(200, 1000)); // simulate API latency
              });
            case 18:
              return _context.abrupt("return", _context.sent);
            case 19:
              return _context.abrupt("return", {
                input: text,
                sid: (0, _ulid.ulid)(),
                completion: null
              });
            case 20:
              _context.next = 22;
              return fetch('https://api.foundation-ui.com/completions', {
                method: 'POST',
                mode: 'cors',
                headers: {
                  'Content-Type': 'application/json',
                  Authorization: "Bearer ".concat(_this.modelId)
                },
                body: JSON.stringify({
                  model: _this.modelId,
                  input: text,
                  session: _this.sessionId,
                  context: context
                })
              });
            case 22:
              res = _context.sent;
              if (!(res.status !== 200)) {
                _context.next = 26;
                break;
              }
              console.error("Couldn't fetch completions");
              return _context.abrupt("return", null);
            case 26:
              _context.next = 28;
              return res.json();
            case 28:
              data = _context.sent;
              _this.fetchCache[text] = data;
              if (((_this$currentCompleti3 = _this.currentCompletion) === null || _this$currentCompleti3 === void 0 ? void 0 : _this$currentCompleti3.sequenceId) === seqId) {
                _this.addCompletion(text, data);
                _this.currentCompletion.completion = data.completion;
              }
              return _context.abrupt("return", data);
            case 32:
            case "end":
              return _context.stop();
          }
        }, _callee);
      }));
      return function (_x, _x2) {
        return _ref.apply(this, arguments);
      };
    }(), 200));
    (0, _defineProperty2["default"])(this, "sendEvents", (0, _lodash2["default"])(function (text, context) {
      fetch('https://api.foundation-ui.com/events', {
        method: 'POST',
        mode: 'cors',
        headers: {
          'Content-Type': 'application/json',
          Authorization: "Bearer ".concat(_this.modelId)
        },
        body: JSON.stringify({
          model: _this.modelId,
          session: _this.sessionId,
          input: text,
          context: context
        })
      }).then(function (res) {
        if (res.status !== 200) {
          console.error('Error sending events');
        }
      })["catch"](function (e) {
        return console.error(e);
      });
    }, 1000, {
      trailing: true
    }));
    this.addCompletion = addCompletion;
    this.modelId = modelId;
    this.mockCompletion = mockCompletion;
  }
  (0, _createClass2["default"])(SmartInputFetcher, [{
    key: "updateCompletion",
    value: function updateCompletion(completionSid, arg) {
      var _this2 = this;
      if (this.mockCompletion) {
        return;
      }

      // reduce chattiness
      var shouldUpdate = arg.seen && !this.alreadySeen.has(completionSid) || arg.accepted && !this.alreadyAccepted.has(completionSid);
      if (!shouldUpdate) {
        return;
      }
      fetch('https://api.foundation-ui.com/completions', {
        method: 'PUT',
        mode: 'cors',
        headers: {
          'Content-Type': 'application/json',
          Authorization: "Bearer ".concat(this.modelId)
        },
        body: JSON.stringify({
          completionSid: completionSid,
          seen: arg.seen,
          accepted: arg.accepted
        })
      }).then(function (res) {
        if (res.status !== 200) {
          console.error("Couldn't update completion");
        }
        if (arg.seen) {
          // only update the set if the update was successful
          _this2.alreadySeen.add(completionSid);
        }
        if (arg.accepted) {
          // only update the set if the update was successful
          _this2.alreadyAccepted.add(completionSid);
        }
      })["catch"](function (e) {
        return console.error(e);
      });
    }
  }]);
  return SmartInputFetcher;
}();
/**
 * Returns a random integer between min (inclusive) and max (inclusive).
 * The value is no lower than min (or the next integer greater than min
 * if min isn't an integer) and no greater than max (or the next integer
 * lower than max if max isn't an integer).
 * Using Math.round() will give you a non-uniform distribution!
 */
exports.SmartInputFetcher = SmartInputFetcher;
function getRandomInt(min, max) {
  min = Math.ceil(min);
  max = Math.floor(max);
  return Math.floor(Math.random() * (max - min + 1)) + min;
}
//# sourceMappingURL=SmartInputFetcher.js.map